@@ -11,9 +11,9 @@
 #define ERATBIG_HPP
 
 #include "Bucket.hpp"
+#include "noinline.hpp"
 #include "MemoryPool.hpp"
 #include "Wheel.hpp"
-#include "types.hpp"
 
 #include <stdint.h>
 #include <vector>
@@ -28,7 +28,7 @@ class EratBig : public Wheel210_t
 {
 public:
   void init(uint64_t, uint64_t, uint64_t);
-  void crossOff(byte_t*);
+  NOINLINE void crossOff(uint8_t*);
   bool enabled() const { return enabled_; }
 private:
   uint64_t maxPrime_ = 0;
@@ -37,9 +37,8 @@ private:
   std::vector<SievingPrime*> sievingPrimes_;
   MemoryPool memoryPool_;
   bool enabled_ = false;
-  void init(uint64_t);
   void storeSievingPrime(uint64_t, uint64_t, uint64_t);
-  void crossOff(byte_t*, Bucket*);
+  NOINLINE void crossOff(uint8_t*, Bucket*);
 };
 
 } // namespace
