@@ -1,7 +1,7 @@
 ///
 /// @file  EratMedium.hpp
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -11,8 +11,8 @@
 #define ERATMEDIUM_HPP
 
 #include "Bucket.hpp"
+#include "noinline.hpp"
 #include "MemoryPool.hpp"
-#include "types.hpp"
 #include "Wheel.hpp"
 
 #include <stdint.h>
@@ -29,23 +29,22 @@ class EratMedium : public Wheel30_t
 public:
   void init(uint64_t, uint64_t, uint64_t);
   bool enabled() const { return enabled_; }
-  void crossOff(byte_t*, uint64_t);
+  NOINLINE void crossOff(uint8_t*, uint64_t);
 private:
   bool enabled_ = false;
   uint64_t maxPrime_ = 0;
   MemoryPool memoryPool_;
   std::array<SievingPrime*, 64> sievingPrimes_;
-  void resetSievingPrimes();
   void storeSievingPrime(uint64_t, uint64_t, uint64_t);
-  void crossOff(byte_t*, byte_t*, Bucket*);
-  void crossOff_7(byte_t*, byte_t*, Bucket*);
-  void crossOff_11(byte_t*, byte_t*, Bucket*);
-  void crossOff_13(byte_t*, byte_t*, Bucket*);
-  void crossOff_17(byte_t*, byte_t*, Bucket*);
-  void crossOff_19(byte_t*, byte_t*, Bucket*);
-  void crossOff_23(byte_t*, byte_t*, Bucket*);
-  void crossOff_29(byte_t*, byte_t*, Bucket*);
-  void crossOff_31(byte_t*, byte_t*, Bucket*);
+  void crossOff(uint8_t*, uint8_t*, Bucket*);
+  NOINLINE void crossOff_7(uint8_t*, uint8_t*, Bucket*);
+  NOINLINE void crossOff_11(uint8_t*, uint8_t*, Bucket*);
+  NOINLINE void crossOff_13(uint8_t*, uint8_t*, Bucket*);
+  NOINLINE void crossOff_17(uint8_t*, uint8_t*, Bucket*);
+  NOINLINE void crossOff_19(uint8_t*, uint8_t*, Bucket*);
+  NOINLINE void crossOff_23(uint8_t*, uint8_t*, Bucket*);
+  NOINLINE void crossOff_29(uint8_t*, uint8_t*, Bucket*);
+  NOINLINE void crossOff_31(uint8_t*, uint8_t*, Bucket*);
 };
 
 } // namespace
