@@ -1,7 +1,7 @@
 ///
 /// @file  EratSmall.hpp
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -11,8 +11,8 @@
 #define ERATSMALL_HPP
 
 #include "Bucket.hpp"
+#include "noinline.hpp"
 #include "Wheel.hpp"
-#include "types.hpp"
 
 #include <stdint.h>
 #include <vector>
@@ -26,17 +26,17 @@ namespace primesieve {
 class EratSmall : public Wheel30_t
 {
 public:
-  static uint64_t getL1Size(uint64_t);
+  static uint64_t getL1CacheSize(uint64_t);
   void init(uint64_t, uint64_t, uint64_t);
-  void crossOff(byte_t*, uint64_t);
+  void crossOff(uint8_t*, uint64_t);
   bool enabled() const { return enabled_; }
 private:
-  uint64_t maxPrime_;
-  uint64_t l1Size_;
+  uint64_t maxPrime_ = 0;
+  uint64_t l1CacheSize_ = 0;
   std::vector<SievingPrime> primes_;
   bool enabled_ = false;
   void storeSievingPrime(uint64_t, uint64_t, uint64_t);
-  void crossOff(byte_t*, byte_t*);
+  NOINLINE void crossOff(uint8_t*, uint8_t*);
 };
 
 } // namespace
