@@ -1,7 +1,7 @@
 ///
 /// @file  MemoryPool.hpp
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -11,46 +11,49 @@
 #define MEMORYPOOL_HPP
 
 #include "Bucket.hpp"
-#include <vector>
+#include "noinline.hpp"
+
+#include <cassert>
 #include <memory>
+#include <vector>
 
 namespace primesieve {
 
 class MemoryPool
 {
 public:
-  void reset(SievingPrime*& sievingPrime);
-  void addBucket(SievingPrime*& sievingPrime);
+  NOINLINE void addBucket(SievingPrime*& sievingPrime);
   void freeBucket(Bucket* bucket);
 
   /// Get the sieving prime's bucket.
   /// For performance reasons we don't keep an array with all
   /// buckets. Instead we find the sieving prime's bucket by
-  /// doing pointer arithmetic using the sieving prime's
-  /// address. Since all buckets are aligned by sizeof(Bucket)
-  /// we calculate the next address that is smaller than the
-  /// sieving prime's address and that is aligned by
-  /// sizeof(Bucket). That's the address of the sieving prime's
-  /// bucket.
+  /// doing pointer arithmetic using the sieving prime's address.
+  /// Since all buckets are aligned by sizeof(Bucket) we
+  /// calculate the next address that is smaller than the sieving
+  /// prime's address and that is aligned by sizeof(Bucket).
+  /// That's the address of the sieving prime's bucket.
   ///
   static Bucket* getBucket(SievingPrime* sievingPrime)
   {
+    assert(sievingPrime != nullptr);
     std::size_t address = (std::size_t) sievingPrime;
     // We need to adjust the address
-    // in case the bucket is full
+    // in case the bucket is full.
     address -= 1;
     address -= address % sizeof(Bucket);
     return (Bucket*) address;
   }
 
-  /// Returns true if the sieving prime's bucket is full.
-  /// Since each bucket's memory is aligned by sizeof(Bucket) we can
-  /// compute the position of the current sieving prime using
-  /// address % sizeof(Bucket).
+  /// Returns true if the sieving prime's bucket is full (or if
+  /// there is no bucket at all i.e. sievingPrime == nullptr).
+  /// Since each bucket's memory is aligned by sizeof(Bucket) we
+  /// can compute the position of the current sieving prime
+  /// using address % sizeof(Bucket).
   ///
   static bool isFullBucket(SievingPrime* sievingPrime)
   {
-    std::size_t address = (std::size_t) sievingPrime;
+    auto address = (std::size_t) sievingPrime;
     return address % sizeof(Bucket) == 0;
   }
 
