@@ -6,7 +6,7 @@
 ///        returns the primes. When there are no more primes left in
 ///        the vector PrimeGenerator generates new primes.
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -17,11 +17,9 @@
 
 #include "Erat.hpp"
 #include "PreSieve.hpp"
-#include "littleendian_cast.hpp"
 #include "SievingPrimes.hpp"
 
 #include <stdint.h>
-#include <array>
 #include <vector>
 
 namespace primesieve {
@@ -31,34 +29,9 @@ class PrimeGenerator : public Erat
 public:
   PrimeGenerator(uint64_t start, uint64_t stop);
   void fill(std::vector<uint64_t>&);
+  void fill(std::vector<uint64_t>& primes, std::size_t* size);
+  static uint64_t maxCachedPrime();
 
-  bool finished() const
-  {
-    return finished_;
-  }
-
-  static uint64_t maxCachedPrime()
-  {
-    return smallPrimes.back();
-  }
-
-  void fill(std::vector<uint64_t>& primes,
-            std::size_t* size)
-  {
-    if (sieveIdx_ >= sieveSize_)
-      if (!sieveSegment(primes, size))
-        return;
-
-    std::size_t i = 0;
-    uint64_t bits = littleendian_cast<uint64_t>(&sieve_[sieveIdx_]);
-    sieveIdx_ += 8;
-
-    for (; bits != 0; i++)
-      primes[i] = nextPrime(&bits, low_);
-
-    *size = i;
-    low_ += 8 * 30;
-  }
 private:
   uint64_t low_ = 0;
   uint64_t sieveIdx_ = ~0ull;
@@ -66,9 +39,6 @@ private:
   PreSieve preSieve_;
   SievingPrimes sievingPrimes_;
   bool isInit_ = false;
-  bool finished_ = false;
-  static const std::array<uint64_t, 64> smallPrimes;
-  static const std::array<uint8_t, 312> primePi;
   std::size_t getStartIdx() const;
   std::size_t getStopIdx() const;
   void initErat();
