@@ -1,7 +1,7 @@
 ///
 /// @file  PrintPrimes.hpp
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -11,8 +11,8 @@
 #define PRINTPRIMES_HPP
 
 #include "Erat.hpp"
+#include "noinline.hpp"
 #include "PrimeSieve.hpp"
-#include "types.hpp"
 
 #include <stdint.h>
 #include <vector>
@@ -29,13 +29,11 @@ class PrintPrimes : public Erat
 {
 public:
   PrintPrimes(PrimeSieve&);
-  void sieve();
+  NOINLINE void sieve();
 private:
-  enum { END = 0xff + 1 };
-  static const uint64_t bitmasks_[6][5];
   uint64_t low_ = 0;
   /// Count lookup tables for prime k-tuplets
-  std::vector<byte_t> kCounts_[6];
+  std::vector<uint8_t> kCounts_[6];
   counts_t& counts_;
   /// Reference to the associated PrimeSieve object
   PrimeSieve& ps_;
