@@ -2,7 +2,7 @@
 /// @file   config.hpp
 /// @brief  primesieve compile time constants.
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -13,12 +13,7 @@
 
 #include <stdint.h>
 
-/// Disable assert() by default
-#if !defined(DEBUG) && !defined(NDEBUG)
-  #define NDEBUG
-#endif
-
-namespace primesieve {
+namespace {
 namespace config {
 
 enum {
@@ -37,20 +32,22 @@ enum {
   ///
   MAX_ALLOC_BYTES = (1 << 20) * 16,
 
-  /// primesieve::iterator caches at least MIN_CACHE_ITERATOR
+  /// iterator::prev_prime() caches at least MIN_CACHE_ITERATOR
   /// bytes of primes. Larger is usually faster but also
   /// requires more memory.
   ///
   MIN_CACHE_ITERATOR = (1 << 20) * 8,
 
-  /// primesieve::iterator maximum cache size in bytes, used
+  /// iterator::prev_prime() maximum cache size in bytes, used
   /// if pi(sqrt(n)) * 8 bytes > MAX_CACHE_ITERATOR.
   ///
   MAX_CACHE_ITERATOR = (1 << 20) * 1024
 };
 
   /// Sieving primes <= (sieveSize in bytes * FACTOR_ERATSMALL)
-  /// are processed in EratSmall objects, speed up ~ 5%.
+  /// are processed in EratSmall. The ideal value for
+  /// FACTOR_ERATSMALL has been determined experimentally by
+  /// running benchmarks near 10^10.
   /// @pre FACTOR_ERATSMALL >= 0 && <= 3
   ///
   /// - For x86-64 CPUs after  2010 use 0.4
@@ -59,13 +56,17 @@ enum {
   ///
   const double FACTOR_ERATSMALL = 0.4;
 
-  /// The formula below ensures that each sieving prime in EratMedium
-  /// has at least 1 multiple occurrence in each segment.
-  /// @pre FACTOR_ERATMEDIUM >= 0 && <= 6
+  /// Sieving primes > (sieveSize in bytes * FACTOR_ERATSMALL)
+  /// and <= (sieveSize in bytes * FACTOR_ERATMEDIUM)
+  /// are processed in EratMedium. The ideal value for
+  /// FACTOR_ERATMEDIUM has been determined experimentally by
+  /// running benchmarks near 10^14.
   ///
-  /// FACTOR_ERATMEDIUM <= 30 numbers per byte / max(wheelFactor)
-  /// FACTOR_ERATMEDIUM = 30 / 6
-  /// FACTOR_ERATMEDIUM = 5.0
+  /// @pre FACTOR_ERATMEDIUM >= 0 && <= 9
+  /// FACTOR_ERATMEDIUM * max(sieveSize) / 30 * 6 + 6 <= max(multipleIndex)
+  /// FACTOR_ERATMEDIUM * 2^22 / 30 * 6 + 6 <= 2^23 - 1
+  /// FACTOR_ERATMEDIUM <= ((2^23 - 7) * 30) / (2^22 * 6)
+  /// FACTOR_ERATMEDIUM <= 9.999991655
   ///
   const double FACTOR_ERATMEDIUM = 5.0;
 
@@ -76,6 +77,6 @@ enum {
   const uint64_t MIN_THREAD_DISTANCE = (uint64_t) 1e7;
 
 } // namespace config
-} // namespace primesieve
+} // namespace
 
 #endif
