@@ -1,16 +1,17 @@
 ///
 /// @file   EratMedium.cpp
-/// @brief  Segmented sieve of Eratosthenes optimized for medium
-///         sieving primes. EratMedium is similar to EratSmall except
-///         that in EratMedium each sieving prime is sorted (by its
+/// @brief  EratMedium is a segmented sieve of Eratosthenes
+///         implementation optimized for medium sieving primes.
+///         EratMedium is similar to EratSmall except that in
+///         EratMedium each sieving prime is sorted (by its
 ///         wheelIndex) after the sieving step. When we then iterate
 ///         over the sorted sieving primes in the next segment the
 ///         initial indirect branch i.e. 'switch (wheelIndex)' is
 ///         predicted correctly by the CPU. This improves performance
-///         by up to 20% for sieving primes that have only a few
+///         by up to 30% for sieving primes that have only a few
 ///         multiple occurrences per segment.
 ///
-/// Copyright (C) 2018 Kim Walisch, <kim.walisch@gmail.com>
+/// Copyright (C) 2019 Kim Walisch, <kim.walisch@gmail.com>
 ///
 /// This file is distributed under the BSD License. See the COPYING
 /// file in the top level directory.
@@ -21,7 +22,6 @@
 #include <primesieve/MemoryPool.hpp>
 #include <primesieve/Wheel.hpp>
 #include <primesieve/primesieve_error.hpp>
-#include <primesieve/types.hpp>
 #include <primesieve/bits.hpp>
 
 #include <stdint.h>
@@ -36,9 +36,9 @@
 #define SORT_SIEVING_PRIME(wheelIndex) \
   sort ## wheelIndex: \
   multipleIndex = (uint64_t) (p - sieveEnd); \
-  sievingPrimes_[wheelIndex]++->set(sievingPrime, multipleIndex, wheelIndex); \
   if (memoryPool_.isFullBucket(sievingPrimes_[wheelIndex])) \
     memoryPool_.addBucket(sievingPrimes_[wheelIndex]); \
+  sievingPrimes_[wheelIndex]++->set(sievingPrime, multipleIndex, wheelIndex); \
   continue;
 
 namespace primesieve {
@@ -53,20 +53,14 @@ void EratMedium::init(uint64_t stop, uint64_t sieveSize, uint64_t maxPrime)
 
   if (sieveSize > maxSieveSize)
     throw primesieve_error("EratMedium: sieveSize > 4096 KiB");
-  if (maxPrime > sieveSize * 6)
-    throw primesieve_error("EratMedium: maxPrime > sieveSize * 6");
+  if (maxPrime > sieveSize * 9)
+    throw primesieve_error("EratMedium: maxPrime > sieveSize * 9");
 
   enabled_ = true;
   maxPrime_ = maxPrime;
+  sievingPrimes_.fill(nullptr);
 
   Wheel::init(stop, sieveSize);
-  resetSievingPrimes();
-}
-
-void EratMedium::resetSievingPrimes()
-{
-  for (SievingPrime*& sievingPrime : sievingPrimes_)
-    memoryPool_.reset(sievingPrime);
 }
 
 /// Add a new sieving prime to EratMedium
@@ -74,24 +68,30 @@ void EratMedium::storeSievingPrime(uint64_t prime, uint64_t multipleIndex, uint6
 {
   assert(prime <= maxPrime_);
   uint64_t sievingPrime = prime / 30;
-  sievingPrimes_[wheelIndex]++->set(sievingPrime, multipleIndex, wheelIndex);
+
   if (memoryPool_.isFullBucket(sievingPrimes_[wheelIndex]))
     memoryPool_.addBucket(sievingPrimes_[wheelIndex]);
+
+  sievingPrimes_[wheelIndex]++->set(sievingPrime, multipleIndex, wheelIndex);
 }
 
-void EratMedium::crossOff(byte_t* sieve, uint64_t sieveSize)
+void EratMedium::crossOff(uint8_t* sieve, uint64_t sieveSize)
 {
-  byte_t* sieveEnd = sieve + sieveSize;
-  auto copy = sievingPrimes_;
-  resetSievingPrimes();
+  // Make a copy of sievingPrimes, then reset it
+  auto lists = sievingPrimes_;
+  sievingPrimes_.fill(nullptr);
+  uint8_t* sieveEnd = sieve + sieveSize;
 
-  // Iterate over all bucket lists.
+  // Iterate over the 64 bucket lists.
   // The 1st list contains sieving primes with wheelIndex = 0.
   // The 2nd list contains sieving primes with wheelIndex = 1.
   // The 3rd list contains sieving primes with wheelIndex = 2.
   // ...
-  for (SievingPrime* sievingPrime : copy)
+  for (SievingPrime* sievingPrime : lists)
   {
+    if (!sievingPrime)
+      continue;
+
     Bucket* bucket = memoryPool_.getBucket(sievingPrime);
     bucket->setEnd(sievingPrime);
 
@@ -100,8 +100,7 @@ void EratMedium::crossOff(byte_t* sieve, uint64_t sieveSize)
     // multiples of its sieving primes.
     while (bucket)
     {
-      if (!bucket->empty())
-        crossOff(sieve, sieveEnd, bucket);
+      crossOff(sieve, sieveEnd, bucket);
       Bucket* processed = bucket;
       bucket = bucket->next();
       memoryPool_.freeBucket(processed);
@@ -109,7 +108,7 @@ void EratMedium::crossOff(byte_t* sieve, uint64_t sieveSize)
   }
 }
 
-void EratMedium::crossOff(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
+void EratMedium::crossOff(uint8_t* sieve, uint8_t* sieveEnd, Bucket* bucket)
 {
   SievingPrime* prime = bucket->begin();
   uint64_t primeType = prime->getWheelIndex() / 8;
@@ -128,7 +127,7 @@ void EratMedium::crossOff(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
 }
 
 /// For sieving primes of type n % 30 == 7
-void EratMedium::crossOff_7(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
+void EratMedium::crossOff_7(uint8_t* sieve, uint8_t* sieveEnd, Bucket* bucket)
 {
   SievingPrime* prime = bucket->begin();
   SievingPrime* end = bucket->end();
@@ -138,7 +137,7 @@ void EratMedium::crossOff_7(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
   {
     uint64_t sievingPrime = prime->getSievingPrime();
     uint64_t multipleIndex = prime->getMultipleIndex();
-    byte_t* p = sieve + multipleIndex;
+    uint8_t* p = sieve + multipleIndex;
 
     uint64_t dist0 = sievingPrime * 6 + 1;
     uint64_t dist1 = sievingPrime * 4 + 1;
@@ -184,7 +183,7 @@ void EratMedium::crossOff_7(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
 }
 
 /// For sieving primes of type n % 30 == 11
-void EratMedium::crossOff_11(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
+void EratMedium::crossOff_11(uint8_t* sieve, uint8_t* sieveEnd, Bucket* bucket)
 {
   SievingPrime* prime = bucket->begin();
   SievingPrime* end = bucket->end();
@@ -194,7 +193,7 @@ void EratMedium::crossOff_11(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
   {
     uint64_t sievingPrime = prime->getSievingPrime();
     uint64_t multipleIndex = prime->getMultipleIndex();
-    byte_t* p = sieve + multipleIndex;
+    uint8_t* p = sieve + multipleIndex;
 
     uint64_t dist0 = sievingPrime * 6 + 2;
     uint64_t dist1 = sievingPrime * 4 + 1;
@@ -240,7 +239,7 @@ void EratMedium::crossOff_11(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
 }
 
 /// For sieving primes of type n % 30 == 13
-void EratMedium::crossOff_13(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
+void EratMedium::crossOff_13(uint8_t* sieve, uint8_t* sieveEnd, Bucket* bucket)
 {
   SievingPrime* prime = bucket->begin();
   SievingPrime* end = bucket->end();
@@ -250,7 +249,7 @@ void EratMedium::crossOff_13(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
   {
     uint64_t sievingPrime = prime->getSievingPrime();
     uint64_t multipleIndex = prime->getMultipleIndex();
-    byte_t* p = sieve + multipleIndex;
+    uint8_t* p = sieve + multipleIndex;
 
     uint64_t dist0 = sievingPrime * 6 + 2;
     uint64_t dist1 = sievingPrime * 4 + 2;
@@ -296,7 +295,7 @@ void EratMedium::crossOff_13(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
 }
 
 /// For sieving primes of type n % 30 == 17
-void EratMedium::crossOff_17(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
+void EratMedium::crossOff_17(uint8_t* sieve, uint8_t* sieveEnd, Bucket* bucket)
 {
   SievingPrime* prime = bucket->begin();
   SievingPrime* end = bucket->end();
@@ -306,7 +305,7 @@ void EratMedium::crossOff_17(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
   {
     uint64_t sievingPrime = prime->getSievingPrime();
     uint64_t multipleIndex = prime->getMultipleIndex();
-    byte_t* p = sieve + multipleIndex;
+    uint8_t* p = sieve + multipleIndex;
 
     uint64_t dist0 = sievingPrime * 6 + 3;
     uint64_t dist1 = sievingPrime * 4 + 3;
@@ -352,7 +351,7 @@ void EratMedium::crossOff_17(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
 }
 
 /// For sieving primes of type n % 30 == 19
-void EratMedium::crossOff_19(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
+void EratMedium::crossOff_19(uint8_t* sieve, uint8_t* sieveEnd, Bucket* bucket)
 {
   SievingPrime* prime = bucket->begin();
   SievingPrime* end = bucket->end();
@@ -362,7 +361,7 @@ void EratMedium::crossOff_19(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
   {
     uint64_t sievingPrime = prime->getSievingPrime();
     uint64_t multipleIndex = prime->getMultipleIndex();
-    byte_t* p = sieve + multipleIndex;
+    uint8_t* p = sieve + multipleIndex;
 
     uint64_t dist0 = sievingPrime * 6 + 4;
     uint64_t dist1 = sievingPrime * 4 + 2;
@@ -408,7 +407,7 @@ void EratMedium::crossOff_19(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
 }
 
 /// For sieving primes of type n % 30 == 23
-void EratMedium::crossOff_23(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
+void EratMedium::crossOff_23(uint8_t* sieve, uint8_t* sieveEnd, Bucket* bucket)
 {
   SievingPrime* prime = bucket->begin();
   SievingPrime* end = bucket->end();
@@ -418,7 +417,7 @@ void EratMedium::crossOff_23(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
   {
     uint64_t sievingPrime = prime->getSievingPrime();
     uint64_t multipleIndex = prime->getMultipleIndex();
-    byte_t* p = sieve + multipleIndex;
+    uint8_t* p = sieve + multipleIndex;
 
     uint64_t dist0 = sievingPrime * 6 + 5;
     uint64_t dist1 = sievingPrime * 4 + 3;
@@ -464,7 +463,7 @@ void EratMedium::crossOff_23(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
 }
 
 /// For sieving primes of type n % 30 == 29
-void EratMedium::crossOff_29(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
+void EratMedium::crossOff_29(uint8_t* sieve, uint8_t* sieveEnd, Bucket* bucket)
 {
   SievingPrime* prime = bucket->begin();
   SievingPrime* end = bucket->end();
@@ -474,7 +473,7 @@ void EratMedium::crossOff_29(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
   {
     uint64_t sievingPrime = prime->getSievingPrime();
     uint64_t multipleIndex = prime->getMultipleIndex();
-    byte_t* p = sieve + multipleIndex;
+    uint8_t* p = sieve + multipleIndex;
 
     uint64_t dist0 = sievingPrime * 6 + 6;
     uint64_t dist1 = sievingPrime * 4 + 4;
@@ -520,7 +519,7 @@ void EratMedium::crossOff_29(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
 }
 
 /// For sieving primes of type n % 30 == 1
-void EratMedium::crossOff_31(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
+void EratMedium::crossOff_31(uint8_t* sieve, uint8_t* sieveEnd, Bucket* bucket)
 {
   SievingPrime* prime = bucket->begin();
   SievingPrime* end = bucket->end();
@@ -530,7 +529,7 @@ void EratMedium::crossOff_31(byte_t* sieve, byte_t* sieveEnd, Bucket* bucket)
   {
     uint64_t sievingPrime = prime->getSievingPrime();
     uint64_t multipleIndex = prime->getMultipleIndex();
-    byte_t* p = sieve + multipleIndex;
+    uint8_t* p = sieve + multipleIndex;
 
     uint64_t dist0 = sievingPrime * 6 + 1;
     uint64_t dist1 = sievingPrime * 4 + 0;
