@@ -30,17 +30,6 @@ using std::unique_ptr;
 
 namespace primesieve {
 
-void MemoryPool::reset(SievingPrime*& sievingPrime)
-{
-  if (!stock_)
-    allocateBuckets();
-
-  Bucket* bucket = stock_;
-  stock_ = stock_->next();
-  bucket->setNext(nullptr);
-  sievingPrime = bucket->begin();
-}
-
 void MemoryPool::addBucket(SievingPrime*& sievingPrime)
 {
   if (!stock_)
@@ -48,10 +37,19 @@ void MemoryPool::addBucket(SievingPrime*& sievingPrime)
 
   Bucket* bucket = stock_;
   stock_ = stock_->next();
+  bucket->setNext(nullptr);
+
+  // In case we add a bucket to the front of a
+  // non empty bucket list we need to set the
+  // next pointer of the new bucket to the bucket
+  // that was previously at the front of the list.
+  if (sievingPrime)
+  {
+    Bucket* old = getBucket(sievingPrime);
+    old->setEnd(sievingPrime);
+    bucket->setNext(old);
+  }
 
-  Bucket* old = getBucket(sievingPrime);
-  old->setEnd(sievingPrime);
-  bucket->setNext(old);
   sievingPrime = bucket->begin();
 }
 
@@ -67,13 +65,13 @@ void MemoryPool::allocateBuckets()
   if (memory_.empty())
     memory_.reserve(128);
 
-  // allocate a large chunk of memory
+  // Allocate a large chunk of memory
   size_t bytes = count_ * sizeof(Bucket);
   char* memory = new char[bytes];
   memory_.emplace_back(unique_ptr<char[]>(memory));
   void* ptr = memory;
 
-  // align pointer address to sizeof(Bucket)
+  // Align pointer address to sizeof(Bucket)
   if (!std::align(sizeof(Bucket), sizeof(Bucket), ptr, bytes))
     throw primesieve_error("MemoryPool: failed to align memory!");
 
